"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_link_1 = require("apollo-link");
const graphql_1 = require("graphql");
const ulid_1 = require("ulid");
const formatMessage_1 = __importDefault(require("../formatMessage"));
class OperationProcessor {
    constructor({ operationTimeout = Infinity } = {}) {
        this.execute = (operation) => {
            return new apollo_link_1.Observable(observer => {
                try {
                    const isSubscription = graphql_1.getOperationAST(typeof operation.query !== 'string'
                        ? operation.query
                        : graphql_1.parse(operation.query), operation.operationName || '').operation === 'subscription';
                    let tmt = null;
                    const id = ulid_1.ulid();
                    const op = {
                        id,
                        isSubscription,
                        observer,
                        operation,
                        clearTimeout: () => {
                            clearTimeout(tmt);
                        },
                        startTimeout: () => {
                            if (this.operationTimeout !== Infinity &&
                                this.operationTimeout !== 0) {
                                tmt = setTimeout(() => {
                                    clearTimeout(tmt);
                                    delete this.executedOperations[id];
                                    observer.error(new Error('Timed out'));
                                }, this.operationTimeout);
                            }
                        },
                    };
                    this.executedOperations[op.id] = op;
                    this.send(op);
                }
                catch (e) {
                    observer.error(e);
                }
            });
        };
        this.processOperationResult = (event) => {
            // if operation is a subscription, just stream a value
            // otherwise stream value and close observable (and remove it from operations)
            const operation = this.executedOperations[event.id];
            if (operation) {
                operation.clearTimeout();
                operation.observer.next(event.payload);
                if (!operation.isSubscription) {
                    delete this.executedOperations[event.id];
                    operation.observer.complete();
                }
            }
        };
        this.start = (socket) => {
            this.socket = socket;
            this.stopped = false;
            // send all pending operations
            this.flushQueuedMessages();
        };
        this.stop = () => {
            this.stopped = true;
            this.socket = null;
        };
        this.send = (operation) => {
            // if is stopped, queue
            if (this.stopped) {
                this.queuedOperations.push(operation);
            }
            else {
                this.sendRaw(operation);
            }
        };
        this.sendRaw = (operation) => {
            const message = {
                id: operation.id,
                payload: Object.assign({}, operation.operation, { query: typeof operation.operation.query !== 'string'
                        ? graphql_1.print(operation.operation.query)
                        : operation.operation.query }),
                type: 'GQL_OP',
            };
            this.socket.send(formatMessage_1.default(message));
            operation.startTimeout();
        };
        this.flushQueuedMessages = () => {
            this.queuedOperations.forEach(this.sendRaw);
            this.queuedOperations = [];
        };
        this.stopped = true;
        this.operationTimeout = operationTimeout;
        this.queuedOperations = [];
        this.executedOperations = {};
        this.socket = null;
    }
}
exports.OperationProcessor = OperationProcessor;
exports.default = OperationProcessor;
